@@ -1,11 +1,11 @@
 # Human friendly input/output in Python.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: June 7, 2014
+# Last Change: June 8, 2014
 # URL: https://humanfriendly.readthedocs.org
 
 # Semi-standard module versioning.
-__version__ = '1.8.4'
+__version__ = '1.8.6'
 
 # Standard library modules.
 import math
@@ -150,7 +150,8 @@ def format_timespan(num_seconds):
     """
     if num_seconds < 60:
         # Fast path.
-        return pluralize(round_number(num_seconds), 'second', 'seconds')
+        rounded_number = round_number(num_seconds, num_seconds < 10)
+        return pluralize(rounded_number, 'second', 'seconds')
     else:
         # Slow path.
         result = []
@@ -274,22 +275,48 @@ class Timer(object):
     Easy to use timer to keep track of long during operations.
     """
 
-    def __init__(self, start_time=None):
+    def __init__(self, start_time=None, resumable=False):
         """
         Remember the time when the :py:class:`Timer` was created.
 
         :param start_time: The start time (a float, defaults to the current time).
+        :param resumable: Create a resumable timer (defaults to ``False``).
         """
-        if start_time is None:
-            start_time = time.time()
-        self.start_time = start_time
+        self.resumable = resumable
+        if self.resumable:
+            self.start_time = 0.0
+            self.total_time = 0.0
+        else:
+            self.start_time = start_time or time.time()
+
+    def __enter__(self):
+        """
+        Start or resume counting elapsed time.
+        """
+        if not self.resumable:
+            raise ValueError("Timer is not resumable!")
+        self.start_time = time.time()
+
+    def __exit__(self, exc_type=None, exc_value=None, traceback=None):
+        """
+        Stop counting elapsed time.
+        """
+        if not self.resumable:
+            raise ValueError("Timer is not resumable!")
+        self.total_time += time.time() - self.start_time
+        self.start_time = 0
 
     @property
     def elapsed_time(self):
         """
-        Get the number of seconds elapsed since the :py:class:`Timer` was created.
+        Get the number of seconds counted so far.
         """
-        return time.time() - self.start_time
+        elapsed_time = 0
+        if self.resumable:
+            elapsed_time += self.total_time
+        if self.start_time:
+            elapsed_time += time.time() - self.start_time
+        return elapsed_time
 
     def __str__(self):
         """
@@ -305,8 +332,17 @@ class Spinner(object):
     happening during long running operations that would otherwise be silent.
     """
 
-    def __init__(self, label=None, stream=sys.stderr):
+    def __init__(self, label=None, total=0, stream=sys.stderr):
+        """
+        Initialize a spinner.
+
+        :param label: The label for the spinner (a string, defaults to ``None``).
+        :param total: The expected number of steps (an integer).
+        :param stream: The output stream to show the spinner on (defaults to
+                       :py:data:`sys.stderr`).
+        """
         self.label = label
+        self.total = total
         self.stream = stream
         self.states = ['-', '\\', '|', '/']
         self.counter = 0
@@ -316,7 +352,7 @@ class Spinner(object):
         except Exception:
             self.interactive = False
 
-    def step(self, label=None):
+    def step(self, progress=0, label=None):
         """
         Advance the spinner by one step without starting a new line, causing
         an animated effect which is very simple but much nicer than waiting
@@ -330,6 +366,8 @@ class Spinner(object):
                 label = label or self.label
                 if not label:
                     raise Exception("No label set for spinner!")
+                elif self.total and progress:
+                    label = "%s: %.2f%%" % (label, progress/(self.total/100.0))
                 self.stream.write("\r %s %s " % (state, label))
                 self.counter += 1
 
